
"      NAME     CalendarView
       AUTHOR   Alexis.Rzewski@mci.com
       URL      
       FUNCTION helps user select a date in a calendar dialog view
       ST-VERSIONS      VisualAge 4.0
       PREREQUISITES    (none)
       CONFLICTS        (none known)
       DISTRIBUTION     world
       VERSION  1.0
       DATE     02-Mar-98

SUMMARY

Implements a dialog-like calendar window with an OK and Cancel button.  
Two comboboxes allow the user to pick a month and year.
A table with each row as a week of seven days allows the user to pick the day of month.

To hook this to a consumer application, add something like the following to the application
that opens the calendar window and is notified with a Date instance when the OK button
is pressed:

finalInitialize
        super finalInitialize.
        self settlementCalendarButton abtWhen: #clicked perform: (#openSettlementCalendar asMessageTo: self).

openSettlementCalendar
        | calendarView |
        calendarView := CalendarView new.
        calendarView 
                abtWhenPrimitive: #value:
                perform: (DirectedMessage new
                                receiver: self;
                                selector: #settlementDate:;
                                arguments: (Array with: nil)).
        calendarView openWindow.

settlementDate: aDate
        (self subpartNamed: 'Settlement Calendar Textfield') object: aDate

                                Alexis Rzewski
"!

Object subclass: #CalendarView
    instanceVariableNames: 'shell main form buttonRowColumn comboboxRowColumn table menuBar menuEdit menuAction popup valueCallback '
    classVariableNames: ''
    poolDictionaries: 'CldtConstants CwConstants EwConstants '!

!CalendarView class publicMethods !

defaultWindowTitle
	^'Calendar'!

monthList
	^#('January' 'February' 'March' 'April' 'May' 'June' 'July' 'August' 'September' 'October' 'November' 'December')
!

new
	^super new initialize
!

weekList
	^(JulianMonth fromMonth: 'February' year: 1996) weekList.!

yearList
	^#('1990' '1991' '1992' '1993' '1994' '1995' '1996' '1997' '1998' '1999')

! !

!CalendarView publicMethods !

buttonConstraints
	buttonRowColumn setValuesBlock: [ :w | w 
		orientation: XmHORIZONTAL;
		height: 32; 
		leftAttachment: XmATTACHFORM;			leftOffset: 2;  
		bottomAttachment: XmATTACHFORM;		bottomOffset: 2;
		topAttachment: XmATTACHNONE;		
		rightAttachment: XmATTACHFORM;		rightOffset: 2 ]!

buttonPressed: widget clientData: clientData callData: callData

	clientData = 'OK' 		
		ifTrue: [
			self signalEvent: #value: with: self date.
			self closeWindow].

	clientData = 'Cancel' 	
		ifTrue: [
			self closeWindow].
!

buttonRowColumn
	^buttonRowColumn

!

callValueCallback
	| callData |

	callData := CwValueCallbackData new.
	callData 	reason: XmCROK;
					value: self date.
	self	callCallbacks: #valueCallback
			callData: callData.



!

cancelButton
	^buttonRowColumn childAt: 2

!

closeWindow
	self shell closeWidget
!

columns
	"Return the collection of columns used in the table list"

	^SevenDayWeek weekdayNames collect: [:weekday |
		 self weekdayColumn: weekday].!

comboboxConstraints
	comboboxRowColumn setValuesBlock: [ :w | w 
		orientation: XmHORIZONTAL;
		height: 28; 
		leftAttachment: XmATTACHFORM;		leftOffset: 2;  
		topAttachment: XmATTACHFORM;		topOffset: 2;
		rightAttachment: XmATTACHFORM;	rightOffset: 2 ]!

comboboxRowColumn
	^comboboxRowColumn

!

convertStringToInteger: aString
	| value |

	"must be non-empty string"
	aString size > 0 ifFalse: [^0].
	
	"characters in string must be all digits"
	(aString conform: [:each | each isDigit])
		ifFalse: [^0].

	value := 0.
	aString doWithIndex: [:eachChar  :index |
		value := value + (eachChar digitValue * (10 raisedTo:  aString size - index))].

	^value

!

createButtons
	| button labels names |
	buttonRowColumn := form createRowColumn: 'buttons' argBlock: nil.
	buttonRowColumn manageChild.
	#( 'OK'  'Cancel' ) do: [ :label |
		button := buttonRowColumn 
							createPushButton: label
							argBlock: [:w | w
								width: 80].
		button addCallback: XmNactivateCallback
			receiver: self
			selector: #buttonPressed:clientData:callData:
			clientData: label.
		button manageChild ] !

createComboboxes
	| combobox labels names |
	comboboxRowColumn := form createRowColumn: 'comboboxes' argBlock: nil.
	comboboxRowColumn manageChild.

	"add the month combobox"
	combobox := comboboxRowColumn 
							createComboBox: 'month' 
							argBlock: [:w | w
								editable: false;
								items: self class monthList].
	combobox addCallback: XmNsingleSelectionCallback
		receiver: self
		selector: #monthSelected:clientData:callData:
		clientData: nil.
	combobox manageChild.

	"add the year combobox"
	combobox := comboboxRowColumn 
							createComboBox: 'year' 
							argBlock: [:w | w
								editable: false;
								items: self class yearList].
	combobox addCallback: XmNsingleSelectionCallback
		receiver: self
		selector: #yearSelected:clientData:callData:
		clientData: nil.
	combobox manageChild.

!

createTable
	table := form 
		createTableList: 'table'
		argBlock: [ :w |	w 
			scrollBarDisplayPolicy: XmASNEEDED;
			selectionPolicy: XmCELLSINGLESELECT;
			cellTabbingPolicy: XmAPPLICATIONDEFINED;
			editable: true;
			headingVisualStyle: XmETCHCELLS;
			rowSeparators: false;
			itemHeight: 18;
			traversalOn: true;
			columns: self columns;
			items: self class weekList].

	table addCallback: XmNcellTabCallback
		receiver: self
		selector: #tableTabbed:clientData:callData:
		clientData: nil.
	table manageChild.!

createWindow
	shell := CwTopLevelShell
		createApplicationShell: self windowTitle
		argBlock: [:w | w
			mwmDecorations: MWMDECORBORDER | MWMDECORTITLE].

	main := shell createMainWindow: 'main' argBlock: nil.
	main manageChild.
	form := main createForm: 'aForm' 
			argBlock: [ :w | w width: 180; height: 204 ].
	form manageChild !

date
	^(self table selectedCell notNil) &
	 (self monthCombobox items includes: self monthString) &
	 (self yearCombobox items includes: self yearIndex printString)
		ifTrue: [Date newDay: self dayOfMonth month: self monthString year: self yearIndex]
		ifFalse: [nil].

!

dayOfMonth
	^self table selectedCell notNil
		ifTrue: [(self table items at: self table selectedCell y) dayAt: self table selectedCell x]
		ifFalse: [0]

!

displayWindow
	self monthCombobox setString: Date today monthName asString.
	self yearCombobox setString: Date today year printString.
	shell realizeWidget.!

form
	"answers a CwForm instance"
	^form

!

initialize
!

main
	"answers a CwMainWindow instance"
	^main

!

monthCombobox
	^comboboxRowColumn childAt: 1

!

monthSelected: widget clientData: clientData callData: callData
	self table items: (JulianMonth fromMonth: callData item year: self yearIndex) weekList.!

monthString
	^self monthCombobox getString!

okButton
	^buttonRowColumn childAt: 1

!

openWindow
	self createWindow.

	self createComboboxes.
	self createTable.
	self createButtons.

	self comboboxConstraints.
	self tableConstraints.
	self buttonConstraints.

	self displayWindow.!

shell
	"answers a CwTopLevelShell instance"
	^shell
!

table
	"answers a EwTableList instance"
	^table

!

tableConstraints
	table setValuesBlock: [ :w | w 
		leftAttachment: XmATTACHFORM;		leftOffset: 2;  
		topAttachment: XmATTACHFORM;		topOffset: 32;
		bottomAttachment: XmATTACHFORM;	 bottomOffset: 36;
		rightAttachment: XmATTACHFORM;	rightOffset: 2 ]!

tableTabbed: widget clientData: clientData callData: callData
	"callData is an instance of EwCellTabCallbackData
	 callData cursoredCell answers aPoint where the cell was when tab was pressed
	 callData tabDirection answers an integer 0 if tabbing forward and 1 if tabbing reverse
	 callData focusWidget answers the widget where focus will go after tabbing"

	| nextWidget | 
	nextWidget := callData tabDirection = 0
		ifTrue: 		[self okButton]
		ifFalse: 	[self yearCombobox].

	callData focusWidget: nextWidget.



!

valueCallback
	valueCallback == nil
		ifTrue: [self valueCallback: (OrderedCollection new: 1)].
	^valueCallback

!

valueCallback: aCollection
	valueCallback := aCollection

!

weekday: widget clientData: clientData callData: callData
	| dayIndex |
	dayIndex := ((Date dayOfWeek: clientData asSymbol) + 1 rem: 7).
	dayIndex := dayIndex = 0 
							ifFalse: [dayIndex]
							ifTrue: 	[7].

	callData value: (callData item dayAt: dayIndex).



!

weekdayColumn: aWeekdayString
	| col |
	col := EwTableColumn new
		editable: true;
		heading: aWeekdayString first asString;
		width: 20;
		horizontalAlignment: XmALIGNMENTCENTER;
		horizontalHeadingAlignment: XmALIGNMENTCENTER;
		verticalSeparatorThickness: 0;
		yourself.
	col
		addCallback: XmNcellValueCallback
			receiver: self
			selector: #weekday:clientData:callData:
			clientData: aWeekdayString.
	^col
!

windowTitle
	^self class defaultWindowTitle
!

yearCombobox
	^comboboxRowColumn childAt: 2

!

yearIndex
	^self convertStringToInteger: (self yearCombobox getString)
!

yearSelected: widget clientData: clientData callData: callData
	| yearIndex |
	yearIndex := self convertStringToInteger: callData item.
	self table items: (JulianMonth fromMonth: self monthString year: yearIndex ) weekList.! !

CalendarView initializeAfterLoad!